import axios, { AxiosResponse } from "axios"

export interface RealtimeListenerOptions {
    pollInterval?: number
}

export class RealtimeListener {
    pollInterval: number; // In seconds
    callback: Function;
    _lastItemId: number;
    _intervalId: number; // Number returned by setInterval
    constructor(callback: Function,  { pollInterval=1 }: RealtimeListenerOptions) {
        if(!callback) {
            throw new Error("Callback is required")
        }
        this.pollInterval = pollInterval
        this.callback = callback
        this._lastItemId = null
    }

    _assert_200(resp: AxiosResponse) {
        if(resp.status != 200) {
            // https://stackoverflow.com/a/66114310
            throw new Error(`Got code ${resp.status} when fetching ${axios.getUri(resp.request)}`);
        }
    }


    async _fetchItemAndDoCallback() {
        let resp = await axios.get(`https://hacker-news.firebaseio.com/v0/item/${this._lastItemId}.json`)

        this._assert_200(resp)

        if(resp.data == null) {
            // Retry as Hacker News items may take some time to propagate
            return this._fetchItemAndDoCallback()
        }

        this.callback(resp.data)
    }

    async _poll() {
        let resp = await axios.get("https://hacker-news.firebaseio.com/v0/maxitem.json")
        this._assert_200(resp)
        const maxItem: number = resp.data // Will be number
        if(this._lastItemId < maxItem) {
            this._lastItemId = maxItem
            await this._fetchItemAndDoCallback()
        }
    }

    start(): RealtimeListener {
        if(this._intervalId != undefined) {
            throw new Error("The listener has already started. To stop it, call stop().")
        }
        this._intervalId = setInterval(this._poll.bind(this), this.pollInterval * 1000)
        this._poll() // Initial poll
        return this
    }

    stop() {
        if(this._intervalId == undefined) {
            throw new Error("Listener isn't running")
        }
        clearInterval(this._intervalId)
        this._intervalId = undefined
    }
}

/* Convinience method for creating a new RealtimeListener and calling start() */
export default function listen(callback: Function, options: RealtimeListenerOptions): RealtimeListener {
    return new RealtimeListener(callback, options).start()
}